@@ -2,7 +2,7 @@
  * This file implements the CPython wrapper of the new NumPy iterator.
  *
  * Copyright (c) 2010 by Mark Wiebe (mwwiebe@gmail.com)
- * The Univerity of British Columbia
+ * The University of British Columbia
  *
  * See LICENSE.txt for the license.
  */
@@ -129,7 +129,7 @@ NpyIter_GlobalFlagsConverter(PyObject *flags_in, npy_uint32 *flags)
             f = f_str;
         }
 
-        if (PyBytes_AsStringAndSize(f, &str, &length) == -1) {
+        if (PyBytes_AsStringAndSize(f, &str, &length) < 0) {
             Py_DECREF(f);
             return 0;
         }
@@ -148,6 +148,11 @@ NpyIter_GlobalFlagsConverter(PyObject *flags_in, npy_uint32 *flags)
                             flag = NPY_ITER_C_INDEX;
                         }
                         break;
+                    case 'i':
+                        if (strcmp(str, "copy_if_overlap") == 0) {
+                            flag = NPY_ITER_COPY_IF_OVERLAP;
+                        }
+                        break;
                     case 'n':
                         if (strcmp(str, "common_dtype") == 0) {
                             flag = NPY_ITER_COMMON_DTYPE;
@@ -238,7 +243,7 @@ npyiter_order_converter(PyObject *order_in, NPY_ORDER *order)
         return ret;
     }
 
-    if (PyBytes_AsStringAndSize(order_in, &str, &length) == -1) {
+    if (PyBytes_AsStringAndSize(order_in, &str, &length) < 0) {
         return 0;
     }
 
@@ -300,7 +305,8 @@ NpyIter_OpFlagsConverter(PyObject *op_flags_in,
             f = f_str;
         }
 
-        if (PyBytes_AsStringAndSize(f, &str, &length) == -1) {
+        if (PyBytes_AsStringAndSize(f, &str, &length) < 0) {
+            PyErr_Clear();
             Py_DECREF(f);
             PyErr_SetString(PyExc_ValueError,
                    "op_flags must be a tuple or array of per-op flag-tuples");
@@ -354,6 +360,11 @@ NpyIter_OpFlagsConverter(PyObject *op_flags_in,
                         break;
                 }
                 break;
+            case 'o':
+                if (strcmp(str, "overlap_assume_elementwise") == 0) {
+                    flag = NPY_ITER_OVERLAP_ASSUME_ELEMENTWISE;
+                }
+                break;
             case 'r':
                 if (length > 4) switch (str[4]) {
                     case 'o':
@@ -683,8 +694,8 @@ npyiter_convert_ops(PyObject *op_in, PyObject *op_flags_in,
             if (op_flags[iop]&(NPY_ITER_READWRITE|NPY_ITER_WRITEONLY)) {
                 fromanyflags |= NPY_ARRAY_UPDATEIFCOPY;
             }
-            ao = (PyArrayObject *)PyArray_FromAny((PyObject *)op[iop],
-                                            NULL, 0, 0, fromanyflags, NULL);
+            ao = (PyArrayObject *)PyArray_FROM_OF((PyObject *)op[iop],
+                                                  fromanyflags);
             if (ao == NULL) {
                 if (PyErr_Occurred() &&
                             PyErr_ExceptionMatches(PyExc_TypeError)) {
@@ -737,7 +748,7 @@ npyiter_init(NewNpyArrayIterObject *self, PyObject *args, PyObject *kwds)
         return -1;
     }
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&OOO&O&OO&i", kwlist,
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&OOO&O&OO&i:nditer", kwlist,
                     &op_in,
                     NpyIter_GlobalFlagsConverter, &flags,
                     &op_flags_in,
@@ -1292,7 +1303,7 @@ npyiter_remove_axis(NewNpyArrayIterObject *self, PyObject *args)
         return NULL;
     }
 
-    if (!PyArg_ParseTuple(args, "i", &axis)) {
+    if (!PyArg_ParseTuple(args, "i:remove_axis", &axis)) {
         return NULL;
     }
 
@@ -1547,6 +1558,9 @@ static PyObject *npyiter_multi_index_get(NewNpyArrayIterObject *self)
         ndim = NpyIter_GetNDim(self->iter);
         self->get_multi_index(self->iter, multi_index);
         ret = PyTuple_New(ndim);
+        if (ret == NULL) {
+            return NULL;
+        }
         for (idim = 0; idim < ndim; ++idim) {
             PyTuple_SET_ITEM(ret, idim,
                     PyInt_FromLong(multi_index[idim]));
@@ -1605,6 +1619,7 @@ npyiter_multi_index_set(NewNpyArrayIterObject *self, PyObject *value)
             PyObject *v = PySequence_GetItem(value, idim);
             multi_index[idim] = PyInt_AsLong(v);
             if (multi_index[idim]==-1 && PyErr_Occurred()) {
+                Py_XDECREF(v);
                 return -1;
             }
         }
@@ -2247,10 +2262,9 @@ npyiter_subscript(NewNpyArrayIterObject *self, PyObject *op)
         return npyiter_seq_item(self, i);
     }
     else if (PySlice_Check(op)) {
-        Py_ssize_t istart = 0, iend = 0, istep = 0;
-        if (PySlice_GetIndices((PySliceObject *)op,
-                            NpyIter_GetNOp(self->iter),
-                            &istart, &iend, &istep) < 0) {
+        Py_ssize_t istart = 0, iend = 0, istep = 0, islicelength;
+        if (NpySlice_GetIndicesEx(op, NpyIter_GetNOp(self->iter),
+                                  &istart, &iend, &istep, &islicelength) < 0) {
             return NULL;
         }
         if (istep != 1) {
@@ -2297,10 +2311,9 @@ npyiter_ass_subscript(NewNpyArrayIterObject *self, PyObject *op,
         return npyiter_seq_ass_item(self, i, value);
     }
     else if (PySlice_Check(op)) {
-        Py_ssize_t istart = 0, iend = 0, istep = 0;
-        if (PySlice_GetIndices((PySliceObject *)op,
-                            NpyIter_GetNOp(self->iter),
-                            &istart, &iend, &istep) < 0) {
+        Py_ssize_t istart = 0, iend = 0, istep = 0, islicelength = 0;
+        if (NpySlice_GetIndicesEx(op, NpyIter_GetNOp(self->iter),
+                                  &istart, &iend, &istep, &islicelength) < 0) {
             return -1;
         }
         if (istep != 1) {
@@ -2413,9 +2426,9 @@ NPY_NO_EXPORT PySequenceMethods npyiter_as_sequence = {
     (binaryfunc)NULL,                       /*sq_concat*/
     (ssizeargfunc)NULL,                     /*sq_repeat*/
     (ssizeargfunc)npyiter_seq_item,         /*sq_item*/
-    (ssizessizeargfunc)npyiter_seq_slice,   /*sq_slice*/
+    (ssizessizeargfunc)NULL,                /*sq_slice*/
     (ssizeobjargproc)npyiter_seq_ass_item,  /*sq_ass_item*/
-    (ssizessizeobjargproc)npyiter_seq_ass_slice,/*sq_ass_slice*/
+    (ssizessizeobjargproc)NULL,             /*sq_ass_slice*/
     (objobjproc)NULL,                       /*sq_contains */
     (binaryfunc)NULL,                       /*sq_inplace_concat */
     (ssizeargfunc)NULL,                     /*sq_inplace_repeat */
